@@ -1,5 +1,6 @@
 /*
- * CU sudo version 1.5.7 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.8 (based on Root Group sudo version 1.1)
+ * Copyright (c) 1994,1996,1998,1999 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -67,7 +68,7 @@
 #include "sudo.h"
 
 #ifndef lint
-static const char rcsid[] = "$Id: logging.c,v 1.106 1998/11/18 04:16:13 millert Exp $";
+static const char rcsid[] = "$Sudo: logging.c,v 1.110 1999/02/03 04:32:15 millert Exp $";
 #endif /* lint */
 
 /*
@@ -288,6 +289,12 @@ void log_error(code)
 		tty, cwd, runas_user);
 	    break;
 
+#ifdef HAVE_KERB5
+	case GLOBAL_KRB5_INIT_ERR:
+	    (void) sprintf(p, "Could not initialize Kerberos V");
+	    break;
+#endif /* HAVE_KERB5 */
+
 	default:
 	    strcat(p, "found a weird error : ");
 	    break;
