@@ -4,7 +4,7 @@ static char yyrcsid[]
 #if __GNUC__ == 2
   __attribute__ ((unused))
 #endif /* __GNUC__ == 2 */
-  = "$OpenBSD: skeleton.c,v 1.11 1998/11/17 06:13:43 dm Exp $";
+  = "$OpenBSD: skeleton.c,v 1.13 1998/11/18 15:45:12 dm Exp $";
 #endif
 #include <stdlib.h>
 #define YYBYACC 1
@@ -19,7 +19,7 @@ static char yyrcsid[]
 #line 2 "parse.yacc"
 
 /*
- *  CU sudo version 1.5.7
+ *  CU sudo version 1.5.8
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -80,7 +80,7 @@ static char yyrcsid[]
 #endif /* !HAVE_STRCASECMP */
 
 #ifndef lint
-static const char rcsid[] = "$Id: parse.yacc,v 1.122 1998/11/20 19:26:16 millert Exp $";
+static const char rcsid[] = "$Sudo: sudo.tab.c,v 1.4 1999/02/11 06:41:31 millert Exp $";
 #endif /* lint */
 
 /*
@@ -165,10 +165,10 @@ void yyerror(s)
 {
     /* save the line the first error occured on */
     if (errorlineno == -1)
-	errorlineno = sudolineno - 1;
+	errorlineno = sudolineno ? sudolineno - 1 : 0;
 #ifndef TRACELEXER
     (void) fprintf(stderr, ">>> sudoers file: %s, line %d <<<\n", s,
-	sudolineno - 1);
+	sudolineno ? sudolineno - 1 : 0);
 #else
     (void) fprintf(stderr, "<*> ");
 #endif
@@ -1418,7 +1418,7 @@ case 48:
 				in_alias = TRUE;
 				/* Allocate space for ga_list if necesary. */
 				expand_ga_list();
-				if (!(ga_list[ga_list_len-1].alias = strdup(yyvsp[0].string))){
+				if (!(ga_list[ga_list_len-1].alias = (char *) strdup(yyvsp[0].string))){
 				    (void) fprintf(stderr,
 				      "%s: cannot allocate memory!\n", Argv[0]);
 				    exit(1);
@@ -1451,7 +1451,7 @@ case 54:
 				in_alias = TRUE;
 				/* Allocate space for ga_list if necesary. */
 				expand_ga_list();
-				if (!(ga_list[ga_list_len-1].alias = strdup(yyvsp[0].string))){
+				if (!(ga_list[ga_list_len-1].alias = (char *) strdup(yyvsp[0].string))){
 				    (void) fprintf(stderr,
 				      "%s: cannot allocate memory!\n", Argv[0]);
 				    exit(1);
